import java.lang.*;
import java.io.*;
/**
 * <b>Programm:</b> Game of Life<br>
 * <b>Copyright:</b> 2001 Andreas Gohr, Frank Schubert<br>
 * <b>License:</b> GPL2 or higher<br>
 * <b>Version:</b> 1.0<br>
 * <br>
 * <b>Info:</b> Diese Klasse realisiert das Consolebasierte Game of Life.
 */
public class Gofl {
  Universum kosmos = null;  //'globales' objekt der klasse Universum
  
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Main
   */
  public static void main( String[] args ) {
    Gofl gofl_text = new Gofl( args );
  }
  
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Gibt uebergebenen String als Integer zurueck
   */
  public int StringToInt(String zeichen) {
    int zahl = 0;
    try {
      zahl = Integer.valueOf( zeichen ).intValue();
    }
    catch( Exception e ) {
      System.err.println("Gofl: Fehler bei String nach Int Umwandlung.");
      printHelp();  //in printHelp wird beendet.
    }
    return zahl;
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Testet die Parameter von Gofl
   * und gibt 'saubere' Parameter zurueck
   */
  public String[] testArgs( String[] args ) {
    String[] parameter = null;
    int parameterLaenge = args.length;
    
    if ( parameterLaenge == 2 ) {
    // Wenn 2 Parameter Dateiname und Generationen zurueck
      parameter = new String[3];
      parameter[0] = "file"; // Zeichen das Datei geoeffnet wird
      parameter[1] = args[0]; // Dateiname
      parameter[2] = args[1]; // Generationen
      return parameter;
    }
    if ( parameterLaenge == 3 ) {
    // Wenn 3 Parameter Dimensionen und Generationen zurueck
      parameter = new String[4];
      parameter[0] = "dim"; // Zeichen das DimX und DimY folgen
      parameter[1] = args[0]; //DimX
      parameter[2] = args[1]; //DimY
      parameter[3] = args[2]; //Generationen
      return parameter;
    }
    if ( ( parameterLaenge > 3 ) || 
         ( parameterLaenge < 2 ) ) {
    // mehr als 3 oder weniger als 2 Parameter -> Abbruch
      printHelp();
    }
    return parameter;
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Liesst Zeile von stdin und gibt diese als String zurueck
   */
  public String liessStdin() {
    String eingabe = null;
    try {
      BufferedReader stdin = 
        new BufferedReader(new InputStreamReader(System.in));
      eingabe = stdin.readLine();  //liesst bis <ENTER> von stdin
    }
    catch( IOException e ) {
      System.out.println("Gofl: E/A-Fehler.");
      printHelp();
    }
    return eingabe;
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Liesst Werte von stdin und fuellt das Universum
   */
  public void leseWerte() {
    String eingabe = null;
    int x = kosmos.getX(); //Dimension aus kosmos lesen
    int y = kosmos.getY(); 
    int i = 0;  //for variablen
    int j = 0;
    
    System.out.println("Gofl: * - belebt, 'anderes Zeichen' - unbelebt.");
    System.out.print("Gofl: Nur " + y );
    System.out.println(" Zeichen pro Zeile werden interpretiert!");
    for (i=0; i < x; i++) {  //liesst Dim. x mal. ('Zeilen')
      eingabe = "";
      while( ( eingabe.compareTo("") == 0) ||
             ( eingabe.length() < y )) {
      // Enter und Zeichenzahl < y wird nicht akzeptiert
        System.out.print("Zeile (" + (i+1) + "):");  //Ausgabe der Zeilennr.
        eingabe = liessStdin();
      }
      for (j=0; j < y; j++) {
        if (eingabe.substring(j,j+1).compareTo("*") == 0) {
	// einzelne Zeichen extrahieren und interpretieren
	  kosmos.bear(i,j);  // wenn * dann amoebe erstellen
	}
	else kosmos.kill(i,j);  // sonst killen
      }
    }
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Konstruktor
   */
  public Gofl(String[] args) {
    int gen = 0;  // Generationen
    String[] parameter=null;
    parameter = testArgs( args ); // Ueberpruefung der Parameter
    
    if ( parameter[0].compareTo( "file" ) == 0 ) { // wenn aus Datei
      gen = StringToInt( parameter[2] );
      try {
        kosmos = new Universum( parameter[1] );  // Konstruktor mit Dateinamen
      }                                          // aufrufen
      catch ( Exception e ) {
        System.err.println("Gofl: Fehler beim Erzeugen des Universums.");
	printHelp();
      }
    }
    if ( parameter[0].compareTo( "dim" ) == 0 ) { // wenn Dimensionen
      gen = StringToInt( parameter[3] );
      int DimX = StringToInt( parameter[1] );  // Dimensionen lesen
      int DimY = StringToInt( parameter[2] );
      try {
        kosmos = new Universum( DimX, DimY );  // Konstruktor mit Dimensions-
      }                                        // angabe aufrufen
      catch( Exception e ) {
        System.err.println("Gofl: Fehler beim Erzeugen des Universums.");
	printHelp();
      }
      leseWerte();  // Methode zum einlesen der Felder aufrufen
    }
    System.out.println( "Generation:" + kosmos.getGen() ); // Ausgabe
    show();                             // der Ausgangskonfiguration 
    for(int z = 1 ; z <= gen ; z++) {  // erste Generation ist Ausgangsconfig!
      System.out.print("Gofl: <ENTER> fuer naechste Generation:");
      liessStdin();
      System.out.println("Generation:" + (kosmos.getGen()+1) ); // deshalb ist letzte gen+1!
      kosmos.evolution();
      show();
    }
    speichernAbfrage();
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Abfrage Dateiname + Speichern des Universums
   */
  public void speichernAbfrage() {
    String eingabe = "";  
    System.out.print("Gofl: Speichern? (<ENTER> - nein):");
    eingabe = liessStdin();
    //Test der Eingabe auf y,Y,j,J dann abfrage des Dateinamens
    if ( ( eingabe.compareTo("y") == 0 ) || 
         ( eingabe.compareTo("Y") == 0 ) ||
         ( eingabe.compareTo("j") == 0 ) || 
	 ( eingabe.compareTo("J") == 0 ) ) {   
      System.out.print("<ENTER> = Life.out (!wird ueberschrieben!)");
      System.out.print(", oder neuer Name:");
      eingabe = liessStdin();
      
      if ( eingabe.compareTo("") == 0 ) eingabe = "Life.out";
      if( kosmos.save(eingabe) ) 
        System.out.println("Gofl: Speichern erfolgreich.");
      else System.out.println("Gofl: Speichern nicht erfolgreich.");
    }
    else System.out.println("Gofl: Okay, es wird nicht gespeichert.");
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Zeigt aktuellen Stand des Universums auf der Console an
   */
  public void show() {
    for ( int i=0 ; i < kosmos.getX() ; i++ ) {
      for ( int j=0 ;  j < kosmos.getY() ; j++ ) {
        if ( kosmos.watch(i,j) ) {
          System.out.print('*');
        } else {
          System.out.print('.');
          }
      }
      System.out.println();
    }
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Zeigt kurze Hilfe an der Console
   */
  public void printHelp() {
    System.err.println("Gofl: Game of Life Textversion");
    System.err.println("  Benutzung:");
    System.err.println("    Gofl 'Dateiname' 'zu berechnende Generationen'");
    System.err.print("    Gofl 'Hhe' 'Breite' ");
    System.err.println("'zu berechnende Generationen'");
    System.err.println();
    System.err.println("    z.B. Gofl Life.in 7");
    System.exit(1);
  }
}

