import java.io.*;
import java.util.*;

/**
 * <b>Programm:</b> Game of Life<br>
 * <b>Copyright:</b> 2001 (c) Frank Schubert, Andreas Gohr<br>
 * <b>License:</b> GPL2 or higher<br>
 * <b>Version:</b> 1.0 <br>
 * <br>
 * <b>Info:</b> Diese Klasse stellt das virtuelle Amoeben-Universum und
 * alle darauf anwendbaren Funktionen zur Verfuegung.
 */
public class Universum {
  private boolean[][] universe;  // das eigentliche Universum, mit Werten
  private int genCount = 1;
///////////////////////////////////////////////////////////////////////////////
	/**
	 * Speichert aktuellen Stand des Universums
	 *
	 * gibt true zurueck bei erfolgreichem Speichern sonst false
	 */
	public boolean save(String filename) {
	  BufferedWriter fileBuffer = null;
	  int DimX = getX(); // Dimensionen
	  int DimY = getY();
	  int i = 0; // for variablen
	  int j = 0;  
	  //Buffer erstellen:
	  try {
	    fileBuffer = new BufferedWriter(new FileWriter(filename));
	  }
	  catch (IOException e) {
	    System.err.println("EA-Fehler bei oeffnen von "+filename+".");
	    return false;
	  }
	  //Schreiben:
	  try {
	    fileBuffer.write(DimX +" "+ DimY); // Dimensionen in erste Zeile
	    fileBuffer.newLine();
	    for (i=0; i<DimX; i++) {
	      for (j=0; j<DimY; j++) {
	        if (watch(i,j)) {  // watch liefert true wenn amoebe auf (i,j)
		  fileBuffer.write('*');
		} else {
		  fileBuffer.write('.');
		}
	      }
	      fileBuffer.newLine();  // Zeilenabschluss
	    }
	    fileBuffer.close();
	  }
	  catch (IOException e) {
	    System.err.println("Fehler beim schreiben von "+filename+".");
	    return false;
	  }
	  return true;
 	} 

///////////////////////////////////////////////////////////////////////////////
	/**
 	 * Erzeugt Universum aus Datei
 	 */
	public Universum(String filename) throws Exception{
	  BufferedReader fileBuffer = null;
	  String zeile = null;
	  StringTokenizer ST = null;
	  int x = 0;
	  int y = 0;
	  
	  try {
	    fileBuffer = new BufferedReader(new FileReader(filename));
	  }
	  catch (IOException e) {
	    System.err.println("EA-Fehler beim Lesen aus Datei " +
	                       filename +".");
	    throw e;  // Fehler an Aufrufer weiterwerfen
	  }
	  //Erste Zeile einlesen:
	  try {
	    zeile = fileBuffer.readLine();
	  }
	  catch (IOException e) {
	    System.err.println("EA-Fehler beim Lesen aus Datei " +
	                        filename +".");
	    throw e;
	  }

	  ST = new StringTokenizer(zeile);  //mit standard Trennzeichen
	  try {
	    x=Integer.valueOf(ST.nextToken()).intValue(); //Dimension lesen
	    y=Integer.valueOf(ST.nextToken()).intValue();
	  }
	  catch (NoSuchElementException e){
	    System.err.println("Datei fehlerhaft");
	    throw e;
	  }
          universe = new boolean[x][y];
	  try {
	    for (int i=0; i<x; i++){
	    //Zeile lesen und alle Zeichen einzeln interpretieren
	      zeile = fileBuffer.readLine();
	      for (int j=0; j<y; j++) {
		if (zeile.charAt(j)=='*') {
		  bear(i,j);
		} else {
		  kill(i,j);
		}
	      }
	    }
	  }
	  catch (IOException e) {
	    System.err.println("Fehler beim Lesen der Datei!");
	    throw e;
	  }
       }

///////////////////////////////////////////////////////////////////////////////
	/**
	 * Erzeugt leeres Universum mit Dimension x*y
	 */
	public Universum(int x, int y) {
	  universe = new boolean[x][y];
	  //Initialisierung des Feldes als leer:
	  for (int i=0; i<x; i++) {
	    for (int j=0; j<y; j++) {
	      kill(i,j);
	    }
	  }
	}

///////////////////////////////////////////////////////////////////////////////
	/**
	 * Erzeugt eine Amoebe in Feld x,y
	 */
	public void bear(int x,int y) {
	  universe[x][y]=true;
	}

///////////////////////////////////////////////////////////////////////////////
	/**
	 * Toetet eine Amoebe in Feld x,y
	 */
	public void kill(int x,int y) {
	  universe[x][y]=false;
	}

///////////////////////////////////////////////////////////////////////////////
	/**
	 * Gibt true fuer ein belebtes Feld und false fuer ein unbelebtes
	 * Feld zurueck
	 */
	public boolean watch(int x,int y) {
	  return universe[x][y];
	}

///////////////////////////////////////////////////////////////////////////////
	/**
	 * Gibt die "Breite" des Universums zurueck
	 */
	public int getX() {
	  return universe.length;
	}

///////////////////////////////////////////////////////////////////////////////
	/**
	 * Gibt die "Hoehe" des Universums zurueck
	 */
	public int getY() {
	  return universe[0].length;
	}

///////////////////////////////////////////////////////////////////////////////
        /**
	 * Liefert die aktuelle Generation zurueck
	 */
	public int getGen(){ 
	  return genCount;  //getGen wird in Gofl und XGofl benutzt
	}
	
///////////////////////////////////////////////////////////////////////////////
	/**
	 * Berechnet die naechste Generation
	 **/
	public void evolution() {
	  int xl = 0;
	  int yl = 0;
	  int n = 0; //Neighbourcounter
	  int DimX = getX(); //Dimension
	  int DimY = getY();
	  int i = 0; //for Variablen
	  int j = 0;
	  int k = 0;
	  int l = 0;
	  
	  //Universum clonen:(im geclonten wird gelesen, im echten geschrieben)
	  boolean[][] mirrorverse = new boolean[DimX][DimY];
	  for(i=0; i < DimX; i++) {
	    for(j=0; j < DimY; j++) {
	      mirrorverse[i][j]=universe[i][j]; //alle Werte clonen
	    }
	  }   
	  for (i=0; i < DimX; i++) { 
	    for (j=0; j < DimY; j++) {
	      //Nachbartest:
	      n=0;
	      for (k=-1; k <= 1; k++) { // um den aktuellen Wert 'herum-
	        for (l=-1; l <= 1; l++) { // drehen'
	      
	          xl=k+i;  //enthaelt Koordinaten der zu testenden umliegenden
	          yl=l+j;  //Amoebe
	          if (xl < 0) xl = xl + DimX; //wenn negativer index -> + DimX
	          if (yl < 0) yl = yl + DimY; //   - " -             -> + DimY
	          if (xl > (DimX - 1) ) xl = xl - DimX; //wenn zu gross 
	          if (yl > (DimY - 1) ) yl = yl - DimY; //Dim abziehen
	          if (!(xl == i && yl == j)) { // sich selbst ausschliessen!
	            if (mirrorverse[xl][yl]) n++;
	          }
		}
	      }
	      //Ende von Nachbartest
	      if (n==3) bear(i,j);  // wenn 3 Nachbarn Amoebe erstellen
	      else if (n != 2) kill(i,j); // wenn != 2 killen
	    }
	  }
	  genCount++; // Generationszaehler erhoehen
	}
///////////////////////////////////////////////////////////////////////////////
}
/* Begruendung der gewaehlten Programmstruktur:
   Game of Life (Gofl) besteht aus 3 Klassen.
   Gofl.java ist die consolebasierte Variante, diese enthaelt Methoden, welche
   ausschliesslich fuer die Console Variante noetig sind. Dazu gehoeren
   Zeicheneingabe sowie String nach int Umwandlung und die Fehlerbehandlung
   an der Console. Gofl instanziert zur Laufzeit ein Objekt der Klasse
   Universum und benutzt dessen Methoden.
   
   Xgofl.java ist die GUI-basierte Variante, diese enthaelt Methoden zur
   Darstellung von einfachen Fenstern, Menues, Buttons und Grafikdateien.
   Eine eigene Fehlerbehandlung mit Fensterausgabe ist auch realisiert.
   Diese Methoden sind in anderen Klassen nicht sinvoll weil sie dort nicht
   verwendet werden koennten. Auch Xgofl instanziert ein Objekt der Klasse
   Universum und benutzt dessen Methoden.
   
   Universum.java enthaelt den eigentlichen Gofl Algorithmus, sowie Datei
   oeffnen und speichern Methoden die von den anderen Klassen benutzt werden.
   
   Aufgrund dieser einfachen Struktur von 2 Darstellungsklassen die eine
   Algorithmenklasse benutzen ist die gewaehlte Struktur von 3 Dateien
   sinnvoll.
*/
   
   
   
   
   
   
   
   