import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * <b>Programm:</b> Game of Life<br>
 * <b>Copyright:</b> 2001 (c) Frank Schubert, Andreas Gohr<br>
 * <b>License:</b> GPL2 or higher<br>
 * <b>Version:</b> 1.0 <br>
 * <br>
 * <b>Info:</b> Diese Klasse realisiert das GUI basierte Game of Life.
 */
	
public class Xgofl extends JFrame
                   implements ActionListener {
  Universum universe = null;
  JPanel kosmos = new JPanel();
  JButton b_gen = null;
  JLabel l_gen = null;
  
  /////////////////////////////////////////////////////////////////////////////  
  /**
   * Konstruktor
   */
  public Xgofl() {
    setTitle("Game of Life");
    setSize(400,400);
    setJMenuBar(new MainMenu(this));
    addWindowListener(new WindowCallback()); //Beenden beim Fensterschliessen

    b_gen = new JButton("Naechste Generation");
    b_gen.setVisible(false);	//Wird auf visible gesetzt wenn Univerum vorh.
    b_gen.setActionCommand("next_gen");
    b_gen.addActionListener(this);

    l_gen = new JLabel("",JLabel.RIGHT);  //Wird erst auf visible gesetzt
    					  //wenn Universum vorhanden
    l_gen.setVisible(false);

    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(kosmos, BorderLayout.CENTER); 
    getContentPane().add(b_gen, BorderLayout.SOUTH);
    getContentPane().add(l_gen, BorderLayout.NORTH);
    
    
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Main
   */
  public static void main(String Args[]) {
    Xgofl xg = new Xgofl();
    xg.setVisible(true);
  }
  //////////////////////////////////////////////////////////////////////////////
  /**
   * Eventhandler
   */
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (cmd=="neu") {
      neu();
    }
    else if (cmd=="laden") {
      dateiOpen();
    }
    else if (cmd=="speichern") {
      dateiSave();
    }
    else if (cmd=="ende") {
      System.exit(0);
    }
    else if (cmd=="about") {
      about();
    }
    else if (cmd=="next_gen"){
      universe.evolution();
      updateUniverse();
    }
    else{	//Feldbuttons
      changeFeld(cmd);
    }
  }
  //////////////////////////////////////////////////////////////////////////////
  /**
   * Erzeugt ein neues leeres Universum
   */
  void neu(){
     int x=0;
     int y=0;
     String sx=null; //einzulesende Dimension
     String sy=null;

     sx = JOptionPane.showInputDialog(this,
     		"Bitte geben Sie die X-Dimension ein",
		"X-Dimension",
		JOptionPane.QUESTION_MESSAGE);
     if (sx == null) return; //Falls cancel gedrueckt wurde

     sy = JOptionPane.showInputDialog(this,
     		"Bitte geben Sie die Y-Dimension ein",
		"Y-Dimension",
		JOptionPane.QUESTION_MESSAGE);
     if (sy == null) return; //Falls cancel gedrueckt wurde

     try {
       x=(Integer.valueOf(sx)).intValue();
       y=(Integer.valueOf(sy)).intValue();
     }
     catch (Exception e){
       fehler("Dimension ungueltig");
       return;
     }

     try {
       universe = new Universum(x,y);
     }
     catch (Exception e){
       fehler("Universum konnte nicht erzeugt werden");
       return;
     }
     b_gen.setVisible(true); //GenerationButton sichtbar machen
     updateUniverse();
  }
  //////////////////////////////////////////////////////////////////////////////
  /**
   * Aendert die Besetzung eines Feldes
   */
  void changeFeld(String feld) {
    int x=0;
    int y=0;
    StringTokenizer ST = new StringTokenizer(feld); //in feld, xy des selbigen
    
    try {
      x=Integer.valueOf(ST.nextToken()).intValue(); //erster Wert x
      y=Integer.valueOf(ST.nextToken()).intValue(); //zweiter y
    }
    catch (NoSuchElementException e){
      fehler("Konnte Buttonname nicht parsen");
      return;
    }

    if (universe.watch(x,y)) { // wenn feld besetzt killen
      universe.kill(x,y);
    }
    else {
      universe.bear(x,y); // wenn unbesetzt neuen erschaffen
    }
    updateUniverse();
  }
  //////////////////////////////////////////////////////////////////////////////
  /**
   * About-Window
   */
  void about() {
    JOptionPane.showMessageDialog(this,
      "2001 (c) Frank Schubert, Andreas Gohr\n\n"+
      "This Software is licensed under the terms of\n"+
      "the GNU General Public License Version 2 or higher",
      "Game of Life",
      JOptionPane.INFORMATION_MESSAGE); 
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Laedt ein Universum aus einer Datei
   */
  void dateiOpen(){
    JFileChooser chooser = new JFileChooser("."); //im aktuellen dir starten
    int returnVal = chooser.showOpenDialog(this);

    if(returnVal == JFileChooser.APPROVE_OPTION) {
      String datei = chooser.getSelectedFile().getPath(); //Pfad lesen
      try{
        universe = new Universum(datei);
      }
      catch(Exception e) {
        fehler(datei+" konnte nicht geladen werden.");
        return;
      }
      updateUniverse(); // wenn erfolgreich geladen, Anzeige updaten
      b_gen.setVisible(true); // Generationszaehler anzeigen
    }
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Speichert ein Universum in einer Datei
   */
  void dateiSave(){
    if (universe == null){  // nicht existierendes Universum abfangen
      fehler("Es existiert noch kein Universum");
      return;
    }
  
    JFileChooser chooser = new JFileChooser(".");
    if (chooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
      String datei = chooser.getSelectedFile().getPath();
      if (! universe.save(datei)) {
        fehler("Universum konnte nicht gespeichert werden!");
        return;
      }
    }
  } 
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Wird aufgerufen wenn das Universum sich geaendert hat und baut die
   * Anzeige neu auf
   */
  public void updateUniverse(){
    int x = universe.getX();
    int y = universe.getY();
    JButton b;
    ImageIcon amoebe = new ImageIcon("smiley.gif");
    
    kosmos.setVisible(false); //Kosmos und Generationen-Label
    l_gen.setVisible(false);  //ausblenden
    
    kosmos.setLayout(new GridLayout(x,y));
    kosmos.removeAll(); //Alle vorhandenen Elemente entfernen
    
    for (int i=0;i<x;i++) {
      for (int j=0;j<y;j++) {
        if (universe.watch(i,j)) {
	  //Falls smiley.gif nicht geladen werden konnte: * nehmen:
	  if (amoebe.getImageLoadStatus() == MediaTracker.COMPLETE) {
	    b = new JButton(amoebe);
	  }
	  else {
	    b = new JButton("*");
	  }
        }else{
	  b = new JButton("");
        }
	b.setActionCommand(i+" "+j);
	b.addActionListener(this);
        kosmos.add(b);
      }
    }
    l_gen.setText("Generation "+universe.getGen()); //GenerationeLabel updaten
    
    kosmos.setVisible(true); //Kosmos und GenerationenLabel
    l_gen.setVisible(true);  //wieder sichtbar machen
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Gibt den Fehler auf STDERR und als Dialog aus
   */
   void fehler(String fehler){
     System.err.println(fehler);
     JOptionPane.showMessageDialog(this,fehler,"Ein Fehler ist aufgetreten",
       JOptionPane.ERROR_MESSAGE);
   }
}//End of Xgofl


///////////////////////////////////////////////////////////////////////////////
/**
 * Sorgt dafuer, dass die Anwendung beendet wird beim schliessen des Fensters
 */
class WindowCallback extends WindowAdapter {
  public void windowClosing(WindowEvent e){
    System.exit(0);
  }
}


///////////////////////////////////////////////////////////////////////////////
/**
 * Erzeugt das Hauptmenu
 */
class MainMenu extends JMenuBar{
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Konstruktor
   */
  public MainMenu(ActionListener l) {
    JMenu m=null;
  
    m = new JMenu("Game of Life");
    m.add(makeMI("Neues Universum","neu",l,'n'));
    m.add(makeMI("Universum laden","laden",l,'l'));
    m.add(makeMI("Universum speichern","speichern",l,'s'));
    m.add(makeMI("Programm beenden","ende",l,'e'));
    add(m);
			
    m = new JMenu("?");
    m.add(makeMI("Info","about",l,'i'));
    add(m);
  }
  /////////////////////////////////////////////////////////////////////////////
  /**
   * Erzeugt ein MenuItem mit den gewuenschten Eigenschaften (Beschriftung,
   * Name, ActionListener, Shortcut)
   */
  private JMenuItem makeMI (String Label, String name,
                            ActionListener listener, char SC){
    JMenuItem mi;
    mi = new JMenuItem(Label,SC);
    mi.setActionCommand(name);
    mi.addActionListener(listener);
    return mi;
  }
}

									  
